//--------------------------------------------------------------
// Press F1 to get help about using script.
// To access an object that is not located in the current class, start the call with Globals.
// When using events and timers be cautious not to generate memoryleaks,
// please see the help for more information.
//---------------------------------------------------------------
	
    /*
	*******************************************************************
	All examples in this support document are only intended
	to improve understanding of the functionality and 
	handling of the equipment. 
	Beijer Electronics AB cannot assume any liability if these
	examples are used in real applications.
	In view of the wide range of applications for this 
	equipment, users must acquire sufficient knowledge 
	themselves in order to ensure that it is correctly used in 
	their specific application. Persons responsible for the 
	application and the equipment must themselves 
	ensure that each application is in compliance with all 
	relevant requirements, standards and legislation in 
	respect to configuration and safety.
	Beijer Electronics AB will accept no liability for any 
	damage incurred during the installation or use of this 
	equipment.
	Beijer Electronics AB prohibits all modification, changes
	or conversion of the equipment.
	*******************************************************************
	*/

namespace Neo.ApplicationFramework.Generated
{
	using System.Windows.Forms;
	using System;
	using System.Drawing;
	using Neo.ApplicationFramework.Tools;
	using Neo.ApplicationFramework.Common.Graphics.Logic;
	using Neo.ApplicationFramework.Controls;
	using Neo.ApplicationFramework.Interfaces;
	using System.Data;
	using System.Data.SqlServerCe;
	using System.IO;
	using System.Reflection;


	public partial class ExportTrendData
	{
		/// <summary>
		/// Member variable to hold a sql connection.
		/// </summary>
		private SqlCeConnection m_SqlConnection;

		/// <summary>
		/// This method takes a table name and tries to export 
		/// the data to a csv file.
		/// </summary>
		/// <param name="tableName">The name of the table.</param>
		/// <returns>A string with the full path to the file if successful.</returns>
		public string ExportTableToCSV(string tableName)
		{
			// Create a full path for the file, using the table name.
			string trendDataCSVFileName = Path.Combine(ExcecutingPath, tableName + ".txt");
			// Check if there already is a file with that name.
			if(File.Exists(trendDataCSVFileName))
			{
				// If it exists, delete it.
				File.Delete(trendDataCSVFileName);	
			}
			
			// Use a try/catch in case there is an exception.
			try
			{
				// Try to fetch the data from the database and put it into a DataSet.
				DataSet resultSet = GetResultSet(tableName);
				// Check that there was a DataSet and that it contains a table.
				if (resultSet != null && resultSet.Tables != null)
				{
					// Create a csv header using the column names of the DataSet.
					string csvHeader = GetCSVHeader(resultSet.Tables[0].Columns);

					// Create the file and use a StreamWriter to fill it with data.
					using (StreamWriter csvFile = File.CreateText(trendDataCSVFileName))
					{
						// Write the first line, containing the csv header.
						csvFile.WriteLine(csvHeader);
						// Iterate through the DataSet rows.
						foreach (DataRow row in resultSet.Tables[0].Rows)
						{
							// Parse each row and write the result to the file.
							csvFile.WriteLine(GetCSVRow(row));
						}
					}
				}
				else
				{
					// If no DataSet was created, or it didn't contain any table,
					// return a empty string.
					return string.Empty;	
				}
			}
			catch (Exception)
			{
				// If anything went wrong, catch the exception and
				// set the full path to an empty string.
				trendDataCSVFileName = string.Empty;
			}
			// Return the full path to the file.
			return trendDataCSVFileName;
		}

		/// <summary>
		/// Get the path to where the project is excecuting.
		/// This differs if you run the project from the pc
		/// or in the terminal.
		/// </summary>
		private string ExcecutingPath
		{
			get { return Path.GetDirectoryName(Assembly.GetExecutingAssembly().ManifestModule.FullyQualifiedName); }
		}

		/// <summary>
		/// Tries to fetch a DataSet from the database.
		/// </summary>
		/// <param name="tableName">The name of the table.</param>
		/// <returns>DataSet containing all the data from the table.</returns>
		private DataSet GetResultSet(string tableName)
		{
			// Variable to hold the data.
			DataSet dataSet = null;
			// Call a method that tries to connect to the database.
			if (ConnectToDataBase())
			{
				// Using try/catch to prevent a crash, should anything go wrong.
				try
				{
					// This is the sql command to excecute. Fetch all data from the given table.
					string sqlString = string.Format("SELECT * FROM {0}", tableName);
					// Create a data adapter to excecute the query and fetch the data.
					// It uses the connection variable.
					SqlCeDataAdapter sqlCeDataAdapter = new SqlCeDataAdapter(sqlString, m_SqlConnection);
					// Initialize the DataSet that is used to hold the result.
					dataSet = new DataSet();
					// Populate the DataSet with the result from the data adapter.
					sqlCeDataAdapter.Fill(dataSet);
				}
				catch (SqlCeException)
				{
					// Catches any exceptions from the database and prevents the 
					// program from terminating unexpectedly.
				}
			}
			
			// Disconnects from the database.
			// This is very important.
			// If you leave connections open, there can be unexpected behaviour,
			// like locks and crashes. There is also a memory cost for every open
			// connection.
			DisconnectFromDataBase();

			// Return the result.
			return dataSet;
		}

		/// <summary>
		/// Tries to connect to the database.
		/// </summary>
		/// <returns>True if successful, otherwise false.</returns>
		private bool ConnectToDataBase()
		{
			// Checks if there already is an active connection. 
			// If so, it returns true right away.
			if (m_SqlConnection != null && m_SqlConnection.State == ConnectionState.Open)
				return true;

			// Create a new sql connection object.
			m_SqlConnection = new SqlCeConnection();
			// Specify the connectionstring, with the full path to the database.
			m_SqlConnection.ConnectionString = string.Format("data source={0}", Path.Combine(ExcecutingPath, "Database.sdf"));

			// Using a try/catch in case something goes wrong.
			// Should the full path to the database be wrong, it will throw an exception.
			// This prevents the termination of the program.
			try 
			{
				// Tries to open the connection.
				m_SqlConnection.Open();
			}
			catch (SqlCeException)
			{
				// If it went wrong, terminate the connection completely.
				DisconnectFromDataBase();
				// Return false to indicate that the connection couldn't be made.
				return false;
			}

			// Checks the state of the connection, returning true if it is open.
			return m_SqlConnection.State == ConnectionState.Open;
		}

		/// <summary>
		/// Disconnects from the database and disposes the connection.
		/// </summary>
		private void DisconnectFromDataBase()
		{
			// Makes sure that the connection isn't already disposed.
			if (m_SqlConnection != null)
			{
				// Checks to see if the connection is open.
				if(m_SqlConnection.State == ConnectionState.Open)
				{
					// Close the connection.
					m_SqlConnection.Close();
				}
				// Dispose the connection object.
				m_SqlConnection.Dispose();
				// Set it to null, so that it is easy to find out
				// if it is in use or not.
				m_SqlConnection = null;
			}
		}

		/// <summary>
		/// Creates a CSV header using the names
		/// of the columns in the result set.
		/// </summary>
		/// <param name="columns">A collection of columns to use for the header.</param>
		/// <returns>A string with the header.</returns>
		private string GetCSVHeader(DataColumnCollection columns)
		{
			// Create the result string, adding the header begin chars.
			string csvHeader = "// ";
			// Iterate through all the columns in the collection.
			foreach (DataColumn column in columns)
			{
				// Add the column name followed by a comma to the result.
				csvHeader += column.ColumnName + ",";
			}
			// Remove the last character (the last comma) 
			// after all the column names have been added to the result.
			csvHeader = csvHeader.Remove(csvHeader.Length - 1, 1);
			// Add the header ends chars.
			csvHeader += " //";
			// Return the result.
			return csvHeader;
		}

		/// <summary>
		/// Parse a DataRow to create a CSV row.
		/// </summary>
		/// <param name="row">The DataRow to parse.</param>
		/// <returns>A CSV row containing the data for the row.</returns>
		private string GetCSVRow(DataRow row)
		{
			// Create a empty result string.
			string csvRow = string.Empty;
			// Iterate through the cells in the row.
			foreach (object cell in row.ItemArray)
			{
				// Add the value of the cell followed by a comma to the result.
				csvRow += cell.ToString() + ",";
			}
			// Return the result.
			return csvRow;
		}

	}
}
